"""
Symbolic matrices

Matrices with symbolic entries.  The underlying representation is a
pointer to a Maxima object.


EXAMPLES::

    sage: matrix(SR, 2, 2, range(4))
    [0 1]
    [2 3]
    sage: matrix(SR, 2, 2, var('t'))
    [t 0]
    [0 t]

Arithmetic::

    sage: -matrix(SR, 2, range(4))
    [ 0 -1]
    [-2 -3]
    sage: m = matrix(SR, 2, [1..4]); sqrt(2)*m
    [  sqrt(2) 2*sqrt(2)]
    [3*sqrt(2) 4*sqrt(2)]    
    sage: m = matrix(SR, 4, [1..4^2])
    sage: m * m
    [ 90 100 110 120]
    [202 228 254 280]
    [314 356 398 440]
    [426 484 542 600]

    sage: m = matrix(SR, 3, [1, 2, 3]); m
    [1]
    [2]
    [3]
    sage: m.transpose() * m
    [14]

Computing inverses::

    sage: M = matrix(SR, 2, var('a,b,c,d'))
    sage: ~M
    [-b*c/((b*c/a - d)*a^2) + 1/a            b/((b*c/a - d)*a)]
    [           c/((b*c/a - d)*a)               -1/(b*c/a - d)]
    sage: (~M*M).simplify_rational()
    [1 0]
    [0 1]
    sage: M = matrix(SR, 3, 3, range(9)) - var('t')
    sage: (~M * M).simplify_rational()
    [1 0 0]
    [0 1 0]
    [0 0 1]

    sage: matrix(SR, 1, 1, 1).inverse()
    [1]        
    sage: matrix(SR, 0, 0).inverse()
    []
    sage: matrix(SR, 3, 0).inverse()
    Traceback (most recent call last):
    ...
    ArithmeticError: self must be a square matrix


Test pickling::

    sage: m = matrix(SR, 2, [sqrt(2), 3, pi, e]); m
    [sqrt(2)       3]
    [     pi       e]
    sage: TestSuite(m).run()

Comparison::

    sage: m = matrix(SR, 2, [sqrt(2), 3, pi, e])
    sage: cmp(m,m)
    0
    sage: cmp(m,3) != 0
    True
    sage: m = matrix(SR,2,[1..4]); n = m^2
    sage: (exp(m+n) - exp(m)*exp(n)).simplify_rational() == 0       # indirect test
    True


Determinant::

    sage: M = matrix(SR, 2, 2, [x,2,3,4])
    sage: M.determinant()
    4*x - 6
    sage: M = matrix(SR, 3,3,range(9))
    sage: M.det()
    0
    sage: t = var('t')
    sage: M = matrix(SR, 2, 2, [cos(t), sin(t), -sin(t), cos(t)])
    sage: M.det()
    sin(t)^2 + cos(t)^2

Permanents::

    sage: M = matrix(SR, 2, 2, [x,2,3,4])
    sage: M.permanent()
    4*x + 6            

Rank::
       
    sage: M = matrix(SR, 5, 5, range(25))
    sage: M.rank()
    2
    sage: M = matrix(SR, 5, 5, range(25)) - var('t')
    sage: M.rank()
    5

    .. warning::

        :meth:`rank` may return the wrong answer if it cannot determine that a
        matrix element that is equivalent to zero is indeed so.

Copying symbolic matrices::

    sage: m = matrix(SR, 2, [sqrt(2), 3, pi, e])
    sage: n = copy(m)
    sage: n[0,0] = sin(1)
    sage: m
    [sqrt(2)       3]
    [     pi       e]
    sage: n
    [sin(1)      3]
    [    pi      e]

Conversion to Maxima::

    sage: m = matrix(SR, 2, [sqrt(2), 3, pi, e])
    sage: m._maxima_()
    matrix([sqrt(2),3],[%pi,%e])

"""


from sage.structure.element cimport ModuleElement, RingElement, Element
from sage.structure.factorization import Factorization

cimport matrix_generic_dense
cimport matrix

cdef maxima

from sage.calculus.calculus import symbolic_expression_from_maxima_string, var_cmp, maxima

cdef class Matrix_symbolic_dense(matrix_generic_dense.Matrix_generic_dense):
    def _new_c(self):
        """
        Called when creating a new matrix.

        EXAMPLES::
        
            sage: matrix(SR,0)   # this implicitly calls _new_c
            []
        """
        cdef Matrix_symbolic_dense M = Matrix_symbolic_dense.__new__(Matrix_symbolic_dense, 0, 0, 0)
        matrix.Matrix.__init__(M, self._parent)
        return M

    def eigenvalues(self, solution_set=False):
        """
        Compute the eigenvalues by solving the characteristic
        polynomial in maxima

        EXAMPLES::
        
            sage: a=matrix(SR,[[1,2],[3,4]])
            sage: a.eigenvalues()
            [-1/2*sqrt(33) + 5/2, 1/2*sqrt(33) + 5/2]

        """
        if solution_set is not False:
            import sage.misc.misc
            sage.misc.misc.deprecation("solution_set parameter is deprecated")
        maxima_evals = self._maxima_(maxima).eigenvalues()._sage_()
        if len(maxima_evals)==0:
            raise ArithmeticError, "could not determine eigenvalues exactly using symbolic matrices; try using a different type of matrix via self.change_ring(), if possible"
        return sum([[eval]*int(mult) for eval,mult in zip(*maxima_evals)],[])

    def eigenvectors_left(self):
        r"""
        Compute the left eigenvectors of a matrix.

        For each distinct eigenvalue, returns a list of the form (e,V,n)
        where e is the eigenvalue, V is a list of eigenvectors forming a
        basis for the corresponding left eigenspace, and n is the
        algebraic multiplicity of the eigenvalue.

        EXAMPLES::

            sage: A = matrix(SR,3,3,range(9)); A
            [0 1 2]
            [3 4 5]
            [6 7 8]
            sage: es = A.eigenvectors_left(); es
            [(-3*sqrt(6) + 6, [(1, -1/5*sqrt(6) + 4/5, -2/5*sqrt(2)*sqrt(3) + 3/5)], 1), (3*sqrt(6) + 6, [(1, 1/5*sqrt(6) + 4/5, 2/5*sqrt(2)*sqrt(3) + 3/5)], 1), (0, [(1, -2, 1)], 1)]
            sage: eval, [evec], mult = es[0]
            sage: delta = eval*evec - evec*A
            sage: abs(abs(delta)) < 1e-10
            abs(sqrt(144/25*(sqrt(2)*sqrt(3) - sqrt(6))^2 + 1/25*(3*(sqrt(6) - 2)*(2*sqrt(2)*sqrt(3) - 3) + 16*sqrt(2)*sqrt(3) + 5*sqrt(6) - 54)^2 + 1/25*(3*(sqrt(6) - 4)*(sqrt(6) - 2) + 14*sqrt(2)*sqrt(3) + 4*sqrt(6) - 42)^2)) < (1.00000000000000e-10)
            sage: abs(abs(delta)).n() < 1e-10
            True

        ::

            sage: A = matrix(SR, 2, 2, var('a,b,c,d'))
            sage: A.eigenvectors_left()
            [(1/2*a + 1/2*d - 1/2*sqrt(a^2 - 2*a*d + 4*b*c + d^2), [(1, -1/2*(a - d + sqrt(a^2 - 2*a*d + 4*b*c + d^2))/c)], 1), (1/2*a + 1/2*d + 1/2*sqrt(a^2 - 2*a*d + 4*b*c + d^2), [(1, -1/2*(a - d - sqrt(a^2 - 2*a*d + 4*b*c + d^2))/c)], 1)]
            sage: es = A.eigenvectors_left(); es
            [(1/2*a + 1/2*d - 1/2*sqrt(a^2 - 2*a*d + 4*b*c + d^2), [(1, -1/2*(a - d + sqrt(a^2 - 2*a*d + 4*b*c + d^2))/c)], 1), (1/2*a + 1/2*d + 1/2*sqrt(a^2 - 2*a*d + 4*b*c + d^2), [(1, -1/2*(a - d - sqrt(a^2 - 2*a*d + 4*b*c + d^2))/c)], 1)]
            sage: eval, [evec], mult = es[0]
            sage: delta = eval*evec - evec*A
            sage: delta.apply_map(lambda x: x.full_simplify())
            (0, 0)

        This routine calls Maxima and can struggle with even small matrices
        with a few variables, such as a `3\times 3` matrix with three variables.
        However, if the entries are integers or rationals it can produce exact
        values in a reasonable time.  These examples create 0-1 matrices from
        the adjacency matrices of graphs and illustrate how the format and type
        of the results differ when the base ring changes.  First for matrices
        over the rational numbers, then the same matrix but viewed as a symbolic
        matrix. ::

            sage: G=graphs.CycleGraph(5)
            sage: am = G.adjacency_matrix()
            sage: spectrum = am.eigenvectors_left()
            sage: qqbar_evalue = spectrum[2][0]
            sage: type(qqbar_evalue)
            <class 'sage.rings.qqbar.AlgebraicNumber'>
            sage: qqbar_evalue
            0.618033988749895?

            sage: am = G.adjacency_matrix().change_ring(SR)
            sage: spectrum = am.eigenvectors_left()
            sage: symbolic_evalue = spectrum[2][0]
            sage: type(symbolic_evalue)
            <type 'sage.symbolic.expression.Expression'>
            sage: symbolic_evalue
            1/2*sqrt(5) - 1/2

            sage: qqbar_evalue == symbolic_evalue
            True

        A slightly larger matrix with a "nice" spectrum. ::

            sage: G=graphs.CycleGraph(6)
            sage: am = G.adjacency_matrix().change_ring(SR)
            sage: am.eigenvectors_left()
            [(-1, [(1, 0, -1, 1, 0, -1), (0, 1, -1, 0, 1, -1)], 2), (1, [(1, 0, -1, -1, 0, 1), (0, 1, 1, 0, -1, -1)], 2), (-2, [(1, -1, 1, -1, 1, -1)], 1), (2, [(1, 1, 1, 1, 1, 1)], 1)]
        """
        from sage.modules.free_module_element import vector
        from sage.all import ZZ

        [evals,mults],evecs=self.transpose()._maxima_(maxima).eigenvectors()._sage_()
        result=[]
        for e,evec,m in zip(evals,evecs,mults):
            result.append((e,[vector(v) for v in evec], ZZ(m)))

        return result

    def eigenvectors_right(self):
        r"""
        Compute the right eigenvectors of a matrix.

        For each distinct eigenvalue, returns a list of the form (e,V,n)
        where e is the eigenvalue, V is a list of eigenvectors forming a
        basis for the corresponding right eigenspace, and n is the
        algebraic multiplicity of the eigenvalue.

        EXAMPLES::

            sage: A = matrix(SR,2,2,range(4)); A
            [0 1]
            [2 3]
            sage: right = A.eigenvectors_right(); right
            [(-1/2*sqrt(17) + 3/2, [(1, -1/2*sqrt(17) + 3/2)], 1), (1/2*sqrt(17) + 3/2, [(1, 1/2*sqrt(17) + 3/2)], 1)]

        The right eigenvectors are nothing but the left eigenvectors of the 
        transpose matrix::

            sage: left  = A.transpose().eigenvectors_left(); left
            [(-1/2*sqrt(17) + 3/2, [(1, -1/2*sqrt(17) + 3/2)], 1), (1/2*sqrt(17) + 3/2, [(1, 1/2*sqrt(17) + 3/2)], 1)]
            sage: right[0][1] == left[0][1]
            True
        """
        return self.transpose().eigenvectors_left()

    def exp(self):
        r"""
        Return the matrix exponential of this matrix $X$, which is the matrix

        .. math::

           e^X = \sum_{k=0}^{\infty} \frac{X^k}{k!}.
           
        This function depends on maxima's matrix exponentiation
        function, which does not deal well with floating point
        numbers.  If the matrix has floating point numbers, they will
        be rounded automatically to rational numbers during the
        computation.

        EXAMPLES::
        
            sage: m = matrix(SR,2, [0,x,x,0]); m
            [0 x]
            [x 0]
            sage: m.exp()
            [1/2*(e^(2*x) + 1)*e^(-x) 1/2*(e^(2*x) - 1)*e^(-x)]
            [1/2*(e^(2*x) - 1)*e^(-x) 1/2*(e^(2*x) + 1)*e^(-x)]
            sage: exp(m)
            [1/2*(e^(2*x) + 1)*e^(-x) 1/2*(e^(2*x) - 1)*e^(-x)]
            [1/2*(e^(2*x) - 1)*e^(-x) 1/2*(e^(2*x) + 1)*e^(-x)]

        Exp works on 0x0 and 1x1 matrices::
        
            sage: m = matrix(SR,0,[]); m
            []
            sage: m.exp()
            []
            sage: m = matrix(SR,1,[2]); m
            [2]
            sage: m.exp()
            [e^2]

        Commuting matrices $m, n$ have the property that
        `e^{m+n} = e^m e^n` (but non-commuting matrices need not)::
        
            sage: m = matrix(SR,2,[1..4]); n = m^2
            sage: m*n
            [ 37  54]
            [ 81 118]
            sage: n*m
            [ 37  54]
            [ 81 118]
            
            sage: a = exp(m+n) - exp(m)*exp(n)
            sage: a.simplify_rational() == 0
            True

        The input matrix must be square::
        
            sage: m = matrix(SR,2,3,[1..6]); exp(m)
            Traceback (most recent call last):
            ...
            ValueError: exp only defined on square matrices

        In this example we take the symbolic answer and make it
        numerical at the end::
        
            sage: exp(matrix(SR, [[1.2, 5.6], [3,4]])).change_ring(RDF)
            [346.557487298 661.734590934]
            [354.500673715 677.424782765]

        Another example involving the reversed identity matrix, which
        we clumsily create::
        
            sage: m = identity_matrix(SR,4); m = matrix(list(reversed(m.rows()))) * x
            sage: exp(m)
            [1/2*(e^(2*x) + 1)*e^(-x)                        0                        0 1/2*(e^(2*x) - 1)*e^(-x)]
            [                       0 1/2*(e^(2*x) + 1)*e^(-x) 1/2*(e^(2*x) - 1)*e^(-x)                        0]
            [                       0 1/2*(e^(2*x) - 1)*e^(-x) 1/2*(e^(2*x) + 1)*e^(-x)                        0]
            [1/2*(e^(2*x) - 1)*e^(-x)                        0                        0 1/2*(e^(2*x) + 1)*e^(-x)]
        
        """
        if not self.is_square():
            raise ValueError, "exp only defined on square matrices"
        if self.nrows() == 0:
            return self
        # Maxima's matrixexp function chokes on floating point numbers
        # so we automatically convert floats to rationals by passing
        # keepfloat: false
        m = self._maxima_(maxima)
        z = maxima('matrixexp(%s), keepfloat: false'%m.name())
        if self.nrows() == 1:
            # We do the following, because Maxima stupidly exp's 1x1
            # matrices into non-matrices!
            z = maxima('matrix([%s])'%z.name())

        return z._sage_()

    def charpoly(self, var='x', algorithm=None):
        """
        Compute the characteristic polynomial of self, using maxima.
        
        EXAMPLES:
            sage: M = matrix(SR, 2, 2, var('a,b,c,d'))
            sage: M.charpoly('t')
            t^2 + (-a - d)*t + a*d - b*c
            sage: matrix(SR, 5, [1..5^2]).charpoly()
            x^5 - 65*x^4 - 250*x^3

        """
        from sage.symbolic.ring import SR
        # Maxima has the definition det(matrix-xI) instead of
        # det(xI-matrix), which is what Sage uses elsewhere.  We
        # correct for the discrepancy.
        cp = self._maxima_(maxima).charpoly(var)._sage_()
        cp = cp.expand().polynomial(None, ring=SR[var])
        if self.nrows() % 2 == 1:
            cp = -cp
        return cp

    def fcp(self, var='x'):
        """
        Return the factorization of the characteristic polynomial of self.

        INPUT:

        - ``var`` - (default: 'x') name of variable of charpoly
            
        EXAMPLES::
    
            sage: a = matrix(SR,[[1,2],[3,4]])
            sage: a.fcp()
            x^2 - 5*x - 2
            sage: [i for i in a.fcp()]
            [(x^2 - 5*x - 2, 1)]
            sage: a = matrix(SR,[[1,0],[0,2]])
            sage: a.fcp()
            (x - 2) * (x - 1)
            sage: [i for i in a.fcp()]
            [(x - 2, 1), (x - 1, 1)]
            sage: a = matrix(SR, 5, [1..5^2]) 
            sage: a.fcp()
            (x^2 - 65*x - 250) * x^3
            sage: list(a.fcp())
            [(x^2 - 65*x - 250, 1), (x, 3)]

        """
        from sage.symbolic.ring import SR
        sub_dict = {var: SR.var(var)}
        return Factorization(self.charpoly(var).subs(**sub_dict).factor_list())

    def is_simplified(self):
        """
        Return True if self is the result of running simplify() on a symbolic
        matrix.  This has the semantics of 'has_been_simplified'.
        
        EXAMPLES::
        
            sage: var('x,y,z')
            (x, y, z)
            sage: m = matrix([[z, (x+y)/(x+y)], [x^2, y^2+2]]); m
            [      z       1]
            [    x^2 y^2 + 2]
            sage: m.is_simplified()
            doctest:...: DeprecationWarning: is_simplified is deprecated
            False
            sage: ms = m.simplify(); ms
            [      z       1]
            [    x^2 y^2 + 2]

            sage: m.is_simplified()
            False
            sage: ms.is_simplified()
            False
        """
        import sage.misc.misc
        sage.misc.misc.deprecation("is_simplified is deprecated")
        return False

    def simplify(self):
        """
        Simplifies self.

        EXAMPLES::
        
            sage: var('x,y,z')
            (x, y, z)
            sage: m = matrix([[z, (x+y)/(x+y)], [x^2, y^2+2]]); m
            [      z       1]
            [    x^2 y^2 + 2]
            sage: m.simplify()
            [      z       1]
            [    x^2 y^2 + 2]            
        """
        return self.parent()([x.simplify() for x in self.list()])
    

    def simplify_trig(self):
        """
        EXAMPLES::
        
            sage: theta = var('theta')
            sage: M = matrix(SR, 2, 2, [cos(theta), sin(theta), -sin(theta), cos(theta)])
            sage: ~M
            [-sin(theta)^2/((sin(theta)^2/cos(theta) + cos(theta))*cos(theta)^2) + 1/cos(theta)                    -sin(theta)/((sin(theta)^2/cos(theta) + cos(theta))*cos(theta))]
            [                    sin(theta)/((sin(theta)^2/cos(theta) + cos(theta))*cos(theta))                                           1/(sin(theta)^2/cos(theta) + cos(theta))]
            sage: (~M).simplify_trig()
            [ cos(theta) -sin(theta)]
            [ sin(theta)  cos(theta)]
        """
        return self._maxima_(maxima).trigexpand().trigsimp()._sage_()

    def simplify_rational(self):
        """
        EXAMPLES::
        
            sage: M = matrix(SR, 3, 3, range(9)) - var('t')
            sage: (~M*M)[0,0]
            -(3*((6/t + 7)/((t - 3/t - 4)*t) + 2/t)*((6/t + 5)/((t - 3/t - 4)*t) + 2/t)/((6/t + 5)*(6/t + 7)/(t - 3/t - 4) - t + 12/t + 8) - 1/t - 3/((t - 3/t - 4)*t^2))*t + 3*(6/t + 7)*((6/t + 5)/((t - 3/t - 4)*t) + 2/t)/((t - 3/t - 4)*((6/t + 5)*(6/t + 7)/(t - 3/t - 4) - t + 12/t + 8)) + 6*((6/t + 5)/((t - 3/t - 4)*t) + 2/t)/((6/t + 5)*(6/t + 7)/(t - 3/t - 4) - t + 12/t + 8) - 3/((t - 3/t - 4)*t)
            sage: expand((~M*M)[0,0])
            1
            sage: (~M * M).simplify_rational()
            [1 0 0]
            [0 1 0]
            [0 0 1]
        """
        return self._maxima_(maxima).fullratsimp()._sage_()

    def factor(self):
        """
        Operates point-wise on each element.
        
        EXAMPLES::
        
            sage: M = matrix(SR, 2, 2, x^2 - 2*x + 1); M
            [x^2 - 2*x + 1             0]
            [            0 x^2 - 2*x + 1]
            sage: M.factor()
            [(x - 1)^2         0]
            [        0 (x - 1)^2]
        """
        return self._maxima_(maxima).factor()._sage_()

    def expand(self):
        """
        Operates point-wise on each element.
        
        EXAMPLES::
        
            sage: M = matrix(2, 2, range(4)) - var('x')
            sage: M*M
            [      x^2 + 2      -2*x + 3]
            [     -4*x + 6 (x - 3)^2 + 2]
            sage: (M*M).expand()
            [       x^2 + 2       -2*x + 3]
            [      -4*x + 6 x^2 - 6*x + 11]
        """
        from sage.misc.misc import attrcall
        return self.apply_map(attrcall('expand'))


    def variables(self):
        """
        Returns the variables of self.

        EXAMPLES::
        
            sage: var('a,b,c,x,y')
            (a, b, c, x, y)
            sage: m = matrix([[x, x+2], [x^2, x^2+2]]); m
            [      x   x + 2]
            [    x^2 x^2 + 2]
            sage: m.variables()
            (x,)
            sage: m = matrix([[a, b+c], [x^2, y^2+2]]); m
            [      a   b + c]
            [    x^2 y^2 + 2]
            sage: m.variables()
            (a, b, c, x, y)
        """
        vars = list(set(sum([op.variables() for op in self.list()], ())))
        vars.sort(var_cmp)
        return tuple(vars)

    def arguments(self):
        """
        Returns a tuple of the arguments that self can take.

        EXAMPLES::
        
            sage: var('x,y,z')
            (x, y, z)
            sage: M = MatrixSpace(SR,2,2)
            sage: M(x).arguments()
            (x,)
            sage: M(x+sin(x)).arguments()
            (x,)
        """
        return self.variables()

    def number_of_arguments(self):
        """
        Returns the number of arguments that self can take.

        EXAMPLES::
        
            sage: var('a,b,c,x,y')
            (a, b, c, x, y)
            sage: m = matrix([[a, (x+y)/(x+y)], [x^2, y^2+2]]); m
            [      a       1]
            [    x^2 y^2 + 2]
            sage: m.number_of_arguments()
            3
        """
        return len(self.variables())

    def __call__(self, *args, **kwargs):
        """
        EXAMPLES::
        
            sage: var('x,y,z')
            (x, y, z)
            sage: M = MatrixSpace(SR,2,2)
            sage: h = M(sin(x)+cos(x))
            sage: h
            [sin(x) + cos(x)               0]
            [              0 sin(x) + cos(x)]
            sage: h(x=1)
            [sin(1) + cos(1)               0]
            [              0 sin(1) + cos(1)]
            sage: h(x=x)
            [sin(x) + cos(x)               0]
            [              0 sin(x) + cos(x)]

            sage: h = M((sin(x)+cos(x)).function(x))
            sage: h
            [sin(x) + cos(x)               0]
            [              0 sin(x) + cos(x)]
            sage: h(1)
            doctest:...: DeprecationWarning: Substitution using function-call syntax and unnamed arguments is deprecated and will be removed from a future release of Sage; you can use named arguments instead, like EXPR(x=..., y=...)
            [sin(1) + cos(1)               0]
            [              0 sin(1) + cos(1)]
            sage: h(x)
            [sin(x) + cos(x)               0]
            [              0 sin(x) + cos(x)]

            sage: f = M([0,x,y,z]); f
            [0 x]
            [y z]
            sage: f.arguments()
            (x, y, z)
            sage: f()
            [0 x]
            [y z]
            sage: f(1)
            [0 1]
            [y z]
            sage: f(1,2)
            [0 1]
            [2 z]
            sage: f(1,2,3)
            [0 1]
            [2 3]
            sage: f(x=1)
            [0 1]
            [y z]
            sage: f(x=1,y=2)
            [0 1]
            [2 z]
            sage: f(x=1,y=2,z=3)
            [0 1]
            [2 3]
            sage: f({x:1,y:2,z:3})
            [0 1]
            [2 3]

            sage: f(1, x=2)
            Traceback (most recent call last):
            ...
            ValueError: args and kwargs cannot both be specified
            sage: f(1,2,3,4)
            Traceback (most recent call last):
            ...
            ValueError: the number of arguments must be less than or equal to 3
        """
        if kwargs and args:
            raise ValueError, "args and kwargs cannot both be specified"

        if len(args) == 1 and isinstance(args[0], dict):
            kwargs = dict([(repr(x[0]), x[1]) for x in args[0].iteritems()])

        if kwargs:
            #Handle the case where kwargs are specified
            new_entries = []
            for entry in self.list():
                try:
                    new_entries.append( entry(**kwargs) )
                except ValueError:
                    new_entries.append(entry)
        else:
            #Handle the case where args are specified

            if args:
                from sage.misc.misc import deprecation
                deprecation("Substitution using function-call syntax and unnamed arguments is deprecated and will be removed from a future release of Sage; you can use named arguments instead, like EXPR(x=..., y=...)")
            #Get all the variables
            variables = list( self.arguments() )

            if len(args) > self.number_of_arguments():
                raise ValueError, "the number of arguments must be less than or equal to %s"%self.number_of_arguments()
            
            new_entries = []
            for entry in self.list():
                try:
                    entry_vars = entry.variables()
                    if len(entry_vars) == 0:
                        if len(args) != 0:
                            new_entries.append( entry(args[0]) )
                        else:
                            new_entries.append( entry )
                        continue
                    else:  
                        indices = [i for i in map(variables.index, entry_vars) if i < len(args)]
                        if len(indices) == 0:
                            new_entries.append( entry )
                        else:
                            new_entries.append( entry(*[args[i] for i in indices]) )
                except ValueError:
                    new_entries.append( entry )

        return self.parent(new_entries)
